#include "zpr.h"

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "ply.h"

/* Macro for checking OpenGL error state */

#define GLERROR                                                    \
    {                                                              \
        GLenum code = glGetError();                                \
        while (code!=GL_NO_ERROR)                                  \
        {                                                          \
            printf("%s\n",(char *) gluErrorString(code));          \
                code = glGetError();                               \
        }                                                          \
    }

int num_elems;
Vertex **vlist;
Face **flist;


/******************************************************************************
Read in a PLY file.
******************************************************************************/

void read_test()
{
	int i, j, k;
	PlyFile *ply;
	int nelems;
	char **elist;
	int file_type;
	float version;
	int nprops;
	PlyProperty **plist;
	char *elem_name;
	int num_comments;
	char **comments;
	int num_obj_info;
	char **obj_info;

	/* open a PLY file for reading */
	ply = ply_open_for_reading("test", &nelems, &elist, &file_type, &version);

	/* print what we found out about the file */
	printf("version %f\n", version);
	printf("type %d\n", file_type);

	/* go through each kind of element that we learned is in the file */
	/* and read them */

	for (i = 0; i < nelems; i++) {

		/* get the description of the first element */
		elem_name = elist[i];
		plist = ply_get_element_description(ply, elem_name, &num_elems, &nprops);

		/* print the name of the element, for debugging */
		printf("element %s %d\n", elem_name, num_elems);

		/* if we're on vertex elements, read them in */
		if (equal_strings("vertex", elem_name)) {

			/* create a vertex list to hold all the vertices */
			vlist = (Vertex **)malloc(sizeof(Vertex *) * num_elems);

			/* set up for getting vertex elements */

			ply_get_property(ply, elem_name, &vert_props[0]);
			ply_get_property(ply, elem_name, &vert_props[1]);
			ply_get_property(ply, elem_name, &vert_props[2]);

			/* grab all the vertex elements */
			for (j = 0; j < num_elems; j++) {

				/* grab and element from the file */
				vlist[j] = (Vertex *)malloc(sizeof(Vertex));
				ply_get_element(ply, (void *)vlist[j]);

				/* print out vertex x,y,z for debugging */
				printf("vertex: %g %g %g\n", vlist[j]->x, vlist[j]->y, vlist[j]->z);
			}
		}

		/* if we're on face elements, read them in */
		if (equal_strings("face", elem_name)) {

			/* create a list to hold all the face elements */
			flist = (Face **)malloc(sizeof(Face *) * num_elems);

			/* set up for getting face elements */

			ply_get_property(ply, elem_name, &face_props[0]);
			ply_get_property(ply, elem_name, &face_props[1]);

			/* grab all the face elements */
			for (j = 0; j < num_elems; j++) {

				/* grab and element from the file */
				flist[j] = (Face *)malloc(sizeof(Face));
				ply_get_element(ply, (void *)flist[j]);

				/* print out face info, for debugging */
				printf("face: %d, list = ", flist[j]->intensity);
				for (k = 0; k < flist[j]->nverts; k++)
					printf("%d ", flist[j]->verts[k]);
				printf("\n");
			}
		}

		/* print out the properties we got, for debugging */
		for (j = 0; j < nprops; j++)
			printf("property %s\n", plist[j]->name);
	}

	/* grab and print out the comments in the file */
	comments = ply_get_comments(ply, &num_comments);
	for (i = 0; i < num_comments; i++)
		printf("comment = '%s'\n", comments[i]);

	/* grab and print out the object information */
	obj_info = ply_get_obj_info(ply, &num_obj_info);
	for (i = 0; i < num_obj_info; i++)
		printf("obj_info = '%s'\n", obj_info[i]);

	/* close the PLY file */
	ply_close(ply);
}

/* Draw axes */
void drawAxes(void)
{
    /* Name-stack manipulation for the purpose of
       selection hit processing when mouse button
       is pressed.  Names are ignored in normal
       OpenGL rendering mode.                    */

   glPushMatrix();
                                /* No name for grey sphere */

      glColor3f(0.3,0.3,0.3);
      glutSolidSphere(0.7, 20, 20);

      glPushMatrix();
      glPushName(1);            /* Red cone is 1 */
         glColor3f(1,0,0);
         glRotatef(90,0,1,0);
         glutSolidCone(0.6, 4.0, 20, 20);
      glPopName();
      glPopMatrix();

      glPushMatrix ();
      glPushName(2);            /* Green cone is 2 */
         glColor3f(0,1,0);
         glRotatef(-90,1,0,0);
         glutSolidCone(0.6, 4.0, 20, 20);
      glPopName();
      glPopMatrix();

      glColor3f(0,0,1);         /* Blue cone is 3 */
      glPushName(3);
         glutSolidCone(0.6, 4.0, 20, 20);
      glPopName();

   glPopMatrix();
}

/* Callback function for drawing */

void display(void)
{
   GLERROR;

   glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
   drawAxes();
   glutSwapBuffers();

   GLERROR;
}

/* Callback function for pick-event handling from ZPR */

void pick(GLint name)
{
   printf("Pick: %d\n",name);
   fflush(stdout);
}

static GLfloat light_ambient[]  = { 0.0, 0.0, 0.0, 1.0 };
static GLfloat light_diffuse[]  = { 1.0, 1.0, 1.0, 1.0 };
static GLfloat light_specular[] = { 1.0, 1.0, 1.0, 1.0 };
static GLfloat light_position[] = { 1.0, 1.0, 1.0, 0.0 };

static GLfloat mat_ambient[]    = { 0.7, 0.7, 0.7, 1.0 };
static GLfloat mat_diffuse[]    = { 0.8, 0.8, 0.8, 1.0 };
static GLfloat mat_specular[]   = { 1.0, 1.0, 1.0, 1.0 };
static GLfloat high_shininess[] = { 100.0 };

/* Entry point */

int main(int argc, char *argv[])
{
    /* Initialise GLUT and create a window */

    glutInit(&argc, argv);
    glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH);
    glutInitWindowSize(400,400);
    glutCreateWindow("GLT Mouse Zoom-Pan-Rotate");

    /* Configure GLUT callback functions */

    glutDisplayFunc(display);

    glScalef(0.25,0.25,0.25);

    /* Configure ZPR module */

    zprInit();
    zprSelectionFunc(drawAxes);     /* Selection mode draw function */
    zprPickFunc(pick);              /* Pick event client callback   */

     /* Initialise OpenGL */

    GLERROR;

    glLightfv(GL_LIGHT0, GL_AMBIENT, light_ambient);
    glLightfv(GL_LIGHT0, GL_DIFFUSE, light_diffuse);
    glLightfv(GL_LIGHT0, GL_SPECULAR, light_specular);
    glLightfv(GL_LIGHT0, GL_POSITION, light_position);

    glMaterialfv(GL_FRONT, GL_AMBIENT, mat_ambient);
    glMaterialfv(GL_FRONT, GL_DIFFUSE, mat_diffuse);
    glMaterialfv(GL_FRONT, GL_SPECULAR, mat_specular);
    glMaterialfv(GL_FRONT, GL_SHININESS, high_shininess);

    glEnable(GL_LIGHTING);
    glEnable(GL_LIGHT0);
    glDepthFunc(GL_LESS);
    glEnable(GL_DEPTH_TEST);
    glEnable(GL_NORMALIZE);
    glEnable(GL_COLOR_MATERIAL);

    GLERROR;

    /* Enter GLUT event loop */

    glutMainLoop();

    return 0;
}

